from typing import (
    List,
)
from xml.etree import cElementTree as eTree

from ._Type import JSON
from ._util import (
    get_time_stamp,
    safe_json_loads,
)


class TableStoreFormatter:

    def __init__(self):
        return

    def get_table_store_picture_dict(
        self,
        table_store_picture,          # type: eTree.Element
    ):
        # type: (...) -> JSON

        table_picture_dict = {
            "Tables": self._get_tables(table_store_picture=table_store_picture),
        }

        return table_picture_dict

    def _get_tables(
        self,
        table_store_picture,        # type: eTree.Element
    ):
        # type: (...) -> List

        tables = []
        for table in table_store_picture.findall(".//Table"):
            table_picture_dict = {
                "id": table.get("id"),
                "name": table.get("name"),
                "type": int(table.get("type") or 1),
                "typeDescr": table.get("typeDescr"),
                "seats": int(table.get("seats") or 1),
                "sector": table.get("sector"),
                "subSector": table.get("subSector"),
                "status": int(table.get("status") or 1),
                "statusDescr": table.get("statusDescr"),
                "serviceId": int(table.get("serviceId")) if table.get("serviceId") else None,
                "businessPeriod": table.get("businessPeriod"),
                "userId": table.get("userId"),
                "serviceSeats": int(table.get("serviceSeats") or 0),
                "startTS": get_time_stamp(table.get("startTS")),
                "lastUpdateTS": get_time_stamp(table.get("lastUpdateTS")),
                "statusHistory": self._get_table_status_history(table.get("statusHistory")),
                "tipAmount": float(table.get("tipAmount") or 0),
                "tipRate": table.get("tipRate"),
                "orders": int(table.get("orders") or 0),
                "Properties": self._get_table_properties(table),
            }

            tables.append(table_picture_dict)

        return tables

    @staticmethod
    def _get_table_status_history(
        status_history,     # type: str
    ):
        # type: (...) -> List

        if status_history is None:
            return []

        status_history = safe_json_loads(status_history)
        histories = []
        for status in status_history:
            table_picture_dict = {
                "status": int(status.get("status") or 1),
                "timestamp": get_time_stamp(status.get("timestamp")),
            }

            histories.append(table_picture_dict)

        return histories

    @staticmethod
    def _get_table_properties(
        table,      # type: eTree.Element
    ):
        # type: (...) -> List

        tables = []
        for table_property in table.findall(".//Properties"):
            occupied_seats = []
            occupied_seats_property = table_property.get("occupiedSeats")
            if occupied_seats_property is not None:
                occupied_seats = occupied_seats_property.split(";")

            table_picture_dict = {
                "mode": table_property.get("MODE"),
                "tipType": table_property.get("TIP_TYPE"),
                "userLevel": int(table_property.get("USER_LEVEL") or 0),
                "occupiedSeats": occupied_seats,
            }

            tables.append(table_picture_dict)

        return tables




import ast
import json
from datetime import datetime
from typing import (
    Optional,
    Dict,
)
from xml.etree import cElementTree as eTree


def get_tender_detail(
    tender_line,  # type: eTree.Element
):
    # type: (...) -> Optional[str]

    tender_detail_json = tender_line.get("tenderDetail")
    if not tender_detail_json:
        return None

    try:
        tender_detail = json.loads(tender_detail_json)
    except TypeError:
        return None

    for value in tender_detail.keys():
        if tender_detail[value] is None:
            tender_detail[value] = ""

    return json.dumps(tender_detail)


def get_time_stamp(
    time_stamp,     # type: str
):
    # type: (...) -> Optional[str]

    if time_stamp:
        try:
            ret_time_stamp = datetime.strptime((time_stamp + "Z"), "%Y-%m-%d %H:%M:%SZ")
        except ValueError:
            try:
                ret_time_stamp = datetime.strptime((time_stamp + "Z"), "%Y-%m-%dT%H:%M:%SZ")
            except ValueError:
                return None
        return datetime.strftime(ret_time_stamp, "%Y-%m-%dT%H:%M:%SZ")

    return None


def safe_json_loads(
    text,       # type: str
):
    # type: (...) -> Dict

    try:
        return json.loads(text)
    except json.JSONDecodeError:
        return ast.literal_eval(text)



from ._OrderFormatter import OrderFormatter
from ._PosModelFormatter import PosModelFormatter
from ._TableFormatter import TableFormatter
from ._TableStoreFormatter import TableStoreFormatter
from ._Type import JSON



import json
from xml.etree import cElementTree as eTree

from ._Type import JSON

class PosModelFormatter:

    def __init__(self):
        return

    def get_pos_model_dict(
        self,       # type: PosModelFormatter
        pos_model,  # type: eTree.Element
    ):
        # type: (...) -> JSON

        pos_state = pos_model.find("PosState")
        language = pos_model.find("Language")
        operator = pos_model.find("Operator")
        current_order = pos_model.find("CurrentOrder")
        working_mode = pos_model.find("WorkingMode")
        used_service = pos_model.find("UsedServices")
        stored_modified = pos_model.find("STORE_MODIFIED")
        pos_model_dict = {
            "posId": pos_model.attrib.get("posId"),
            "posState": self._get_attribs(pos_state),
            "language": self._get_attribs(language),
            "operator": self._get_attribs(operator),
            "cashDrawerStatus": self._get_text(pos_model, "CashDrawerStatus"),
            "currentOrder": self._get_attribs(current_order),
            "customs": self._group_same_tag("Custom", pos_model),
            "workingMode": self._get_attribs(working_mode),
            "usedServices": self._group_same_tag("Service", used_service),
        }

        if stored_modified:
            tables = stored_modified.find("Tables")
            pos_model_dict["tables"] = self._group_same_tag("Table", tables)

        return pos_model_dict

    @staticmethod
    def _get_text(
        element,  # type: eTree.Element
        tag,      # type: str
    ):
        # type: (...) -> str

        if element is None:
            return ""

        found = element.find(tag)
        if not found:
            return ""

        return found.text.strip()

    @staticmethod
    def _get_attribs(
        element,  # type: eTree.Element
    ):
        # type: (...) -> JSON

        if element is None:
            return {}

        result = {}
        for key, value in element.attrib.items():
            try:
                result[key] = json.loads(value)

            except (Exception,):
                result[key] = value

        return result

    @staticmethod
    def _group_same_tag(
        tag,        # type: str
        element,    # type: eTree.Element
    ):
        # type: (...) -> JSON

        if element is None:
            return []

        group = []
        for custom_elem in element.findall(tag):
            attrs = PosModelFormatter._get_attribs(custom_elem)
            group.append(attrs)

        return group
